==> unicore-ucc6-5.0.0-1.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/helpers/StorageFactoryLister.java <==
package de.fzj.unicore.ucc.helpers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.StorageFactory;
import de.fzj.unicore.uas.client.StorageFactoryClient;
import de.fzj.unicore.ucc.InaccessibleServiceException;
import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.wsrflite.xmlbeans.client.IRegistryQuery;
import de.fzj.unicore.wsrflite.xmlbeans.client.RegistryClient;
import de.fzj.unicore.wsrflite.xmlbeans.client.Resources;
import eu.unicore.security.util.client.IClientProperties;

/**
 * Provides access to the storage factories in the registry.<br/>
 * 
 * The #list() and {@link #iterator()} methods will run over all the available
 * and accessible SMF services. If you need your own availability checks, and/or you
 * want to perform a task on each SMF, use the {@link #runOnEachSMF(GridTask)} method,
 * providing a custom {@link GridTask}.
 * 
 * @see GridTask
 * @see RegistryClient#listAccessibleServices(javax.xml.namespace.QName)
 * @author schuller
 */
public class StorageFactoryLister implements Iterable<StorageFactoryClient>{

	private final IRegistryQuery registry;
	
	private final IClientProperties sec;
	
	private String[] blacklistPatterns=new String[0];
	
	private int runningLimit=10;
	
	private final MessageWriter msg;
	
	private StorageFactoryFilter filter;
	
	public StorageFactoryLister(IRegistryQuery registry, IClientProperties sec, MessageWriter msg, StorageFactoryFilter filter){
		this.registry=registry;
		this.sec=sec;
		this.msg=msg;
		this.filter=filter;
	}
	
	public StorageFactoryLister(IRegistryQuery registry, IClientProperties sec, MessageWriter msg){
		this(registry, sec, msg, null);
	}
	
	public void setBlacklist(String[] blacklist){
		this.blacklistPatterns=blacklist;
	}
	/**
	 * return a list of all accessible target system factories, 
	 * providing a pre-initialised StorageFactoryClient per target system factory. 
	 * A black list will be honored.
	 * @return
	 */
	public List<StorageFactoryClient> list(){
		List<StorageFactoryClient> sfs=new ArrayList<StorageFactoryClient>();
		try{
			List<Future<StorageFactoryClient>> tasks=runOnEachSMF(new GetStorageFactoryClientTask(filter));
			while(tasks.size()>0){
				Future<StorageFactoryClient>task=tasks.get(0);
				try{
					StorageFactoryClient res=task.get(500, TimeUnit.MILLISECONDS);
					if(res!=null){
						sfs.add(res);
						tasks.remove(0);
					}
				}catch(TimeoutException te){/*no worries will retry*/}
				catch(ExecutionException ee){
					if(! (ee.getCause() instanceof InaccessibleServiceException)){
						String message="Error accessing target system factory.";
						msg.error(message,ee.getCause());
					}
					tasks.remove(0);
				}			
			}
		}catch(Exception e){
			String message="Can't create list of target systems.";
			msg.error(message,e);
		}
		return sfs;
	}
	
	/**
	 * return an iterator over the available target system factories, providing a
	 * pre-initialised StorageFactoryClient per instance
	 * <br/>
	 * in case of errors, an empty iterator is returned
	 * 
	 * @see de.fzj.unicore.uas.client.StorageFactoryClient
	 */
	public Iterator<StorageFactoryClient> iterator(){
		return list().iterator();
	}

	/**
	 * asynchronously execute a {@link GridTask} on each StorageFactory service.
	 * <br/>It will honor a possible blacklist.
	 * @see GridTaskTarget
	 * @return a list of {@link Future} classes for getting the results of the tasks
	 */
	public <T> List<Future<T>>runOnEachSMF(GridTask<T>task)throws Exception{
		List<Future<T>>result=new ArrayList<Future<T>>();
		final List<EndpointReferenceType> smfs=registry.listServices(StorageFactory.SMF_PORT);
		for(EndpointReferenceType epr: smfs){
			if(isBlacklisted(epr)){
				msg.verbose("Ignoring blacklisted site "+epr.getAddress().getStringValue());
				continue;
			}
			GridTaskTarget<T> worker=new GridTaskTarget<T>(sec,registry,msg);
			worker.setEpr(epr);
			worker.setTask(task);
			
			boolean submittedTask;
			do{
				submittedTask=false;
				int running=GridTaskTarget.getRunningTasks();
				if(running<runningLimit){
					result.add(Resources.getExecutorService().submit(worker));
					submittedTask=true;
				}else{
					Thread.sleep(500);
				}
			}while(!submittedTask);
			
		}
		return result;
	}
	
	protected boolean isBlacklisted(EndpointReferenceType epr){
		boolean result=false;
		String url=epr.getAddress().getStringValue();
		for(String pattern: blacklistPatterns){
			if(url.contains(pattern))return true;
		}
		return result;
	}
	
	/**
	 * simple GridTask that just creates a StorageFactoryClient for each target epr and checks
	 * its responsiveness and accessibility by calling getCurrentTime()
	 */
	public static class GetStorageFactoryClientTask implements GridTask<StorageFactoryClient>{

		private final StorageFactoryFilter filter;
		
		public GetStorageFactoryClientTask(){
			this(null);
		}
		
		public GetStorageFactoryClientTask(StorageFactoryFilter filter){
			this.filter=filter;
		}
		
		private StorageFactoryClient createClient(GridTaskTarget<StorageFactoryClient> worker)throws Exception {
			EndpointReferenceType epr=worker.getEpr();
			return new StorageFactoryClient(epr.getAddress().getStringValue(),epr,worker.getSecurity());
		}

		public StorageFactoryClient run(GridTaskTarget<StorageFactoryClient> target) throws Exception {
			StorageFactoryClient client=createClient(target);
			client.getCurrentTime();
			return filter!=null? (filter.accept(client)?client:null) : client;
		}
		
	}
	
	/**
	 * allows to filter the available SMF's
	 *
	 * @author schuller
	 */
	public static interface StorageFactoryFilter{
		
		public boolean accept(StorageFactoryClient smf);
	
	}
	
}

==> unicore-ucc6-6.0.0-0.src.tar.gz: ucc-core/src/main/java/de/fzj/unicore/ucc/helpers/StorageFactoryLister.java <==
package de.fzj.unicore.ucc.helpers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;

import org.w3.x2005.x08.addressing.EndpointReferenceType;

import de.fzj.unicore.uas.StorageFactory;
import de.fzj.unicore.uas.client.StorageFactoryClient;
import de.fzj.unicore.ucc.InaccessibleServiceException;
import de.fzj.unicore.ucc.MessageWriter;
import de.fzj.unicore.wsrflite.xmlbeans.client.IRegistryQuery;
import de.fzj.unicore.wsrflite.xmlbeans.client.RegistryClient;
import de.fzj.unicore.wsrflite.xmlbeans.client.Resources;
import eu.unicore.util.httpclient.IClientConfiguration;

/**
 * Provides access to the storage factories in the registry.<br/>
 * 
 * The #list() and {@link #iterator()} methods will run over all the available
 * and accessible SMF services. If you need your own availability checks, and/or you
 * want to perform a task on each SMF, use the {@link #runOnEachSMF(GridTask)} method,
 * providing a custom {@link GridTask}.
 * 
 * @see GridTask
 * @see RegistryClient#listAccessibleServices(javax.xml.namespace.QName)
 * @author schuller
 */
public class StorageFactoryLister implements Iterable<StorageFactoryClient>{

	private final IRegistryQuery registry;
	
	private final IClientConfiguration sec;
	
	private String[] blacklistPatterns=new String[0];
	
	private int runningLimit=10;
	
	private final MessageWriter msg;
	
	private final StorageFactoryFilter filter;
	
	public StorageFactoryLister(IRegistryQuery registry, IClientConfiguration sec, MessageWriter msg, StorageFactoryFilter filter){
		this.registry=registry;
		this.sec=sec;
		this.msg=msg;
		this.filter=filter;
	}
	
	public StorageFactoryLister(IRegistryQuery registry, IClientConfiguration sec, MessageWriter msg){
		this(registry, sec, msg, null);
	}
	
	public void setBlacklist(String[] blacklist){
		this.blacklistPatterns=blacklist;
	}
	/**
	 * return a list of all accessible target system factories, 
	 * providing a pre-initialised StorageFactoryClient per target system factory. 
	 * A black list set via {@link #setBlacklist(String[])} will be honored.
	 */
	public List<StorageFactoryClient> list(){
		List<StorageFactoryClient> sfs=new ArrayList<StorageFactoryClient>();
		try{
			List<Future<StorageFactoryClient>> tasks=runOnEachSMF(new GetStorageFactoryClientTask(filter));
			while(tasks.size()>0){
				Future<StorageFactoryClient>task=tasks.get(0);
				try{
					StorageFactoryClient res=task.get(500, TimeUnit.MILLISECONDS);
					if(res!=null){
						sfs.add(res);
						tasks.remove(0);
					}
					else{
						tasks.remove(0);
					}
				}catch(TimeoutException te){/*no worries will retry*/}
				catch(ExecutionException ee){
					if(! (ee.getCause() instanceof InaccessibleServiceException)){
						String message="Error accessing target system factory.";
						msg.error(message,ee.getCause());
					}
					tasks.remove(0);
				}			
			}
		}catch(Exception e){
			String message="Can't create list of target systems.";
			msg.error(message,e);
		}
		return sfs;
	}
	
	/**
	 * return an iterator over the available target system factories, providing a
	 * pre-initialised StorageFactoryClient per instance
	 * <br/>
	 * in case of errors, an empty iterator is returned
	 * 
	 * @see de.fzj.unicore.uas.client.StorageFactoryClient
	 */
	public Iterator<StorageFactoryClient> iterator(){
		return list().iterator();
	}

	/**
	 * asynchronously execute a {@link GridTask} on each StorageFactory service.
	 * <br/>It will honor a possible blacklist.
	 * @see GridTaskTarget
	 * @return a list of {@link Future} classes for getting the results of the tasks
	 */
	public <T> List<Future<T>>runOnEachSMF(GridTask<T>task)throws Exception{
		List<Future<T>>result=new ArrayList<Future<T>>();
		final List<EndpointReferenceType> smfs=registry.listServices(StorageFactory.SMF_PORT);
		for(EndpointReferenceType epr: smfs){
			if(filter!=null && !filter.accept(epr)){
				continue;
			}
			if(isBlacklisted(epr)){
				msg.verbose("Ignoring blacklisted site "+epr.getAddress().getStringValue());
				continue;
			}
			GridTaskTarget<T> worker=new GridTaskTarget<T>(sec,registry,msg);
			worker.setEpr(epr);
			worker.setTask(task);
			
			boolean submittedTask;
			do{
				submittedTask=false;
				int running=GridTaskTarget.getRunningTasks();
				if(running<runningLimit){
					result.add(Resources.getExecutorService().submit(worker));
					submittedTask=true;
				}else{
					Thread.sleep(500);
				}
			}while(!submittedTask);
			
		}
		return result;
	}
	
	protected boolean isBlacklisted(EndpointReferenceType epr){
		boolean result=false;
		String url=epr.getAddress().getStringValue();
		for(String pattern: blacklistPatterns){
			if(url.contains(pattern))return true;
		}
		return result;
	}
	
	/**
	 * simple GridTask that just creates a StorageFactoryClient for each target epr and checks
	 * its responsiveness and accessibility by calling getCurrentTime()
	 */
	public static class GetStorageFactoryClientTask implements GridTask<StorageFactoryClient>{

		private final StorageFactoryFilter filter;
		
		public GetStorageFactoryClientTask(){
			this(null);
		}
		
		public GetStorageFactoryClientTask(StorageFactoryFilter filter){
			this.filter=filter;
		}
		
		private StorageFactoryClient createClient(GridTaskTarget<StorageFactoryClient> worker)throws Exception {
			EndpointReferenceType epr=worker.getEpr();
			return new StorageFactoryClient(epr.getAddress().getStringValue(),epr,worker.getSecurity());
		}

		public StorageFactoryClient run(GridTaskTarget<StorageFactoryClient> target) throws Exception {
			StorageFactoryClient client=createClient(target);
			client.getCurrentTime();
			return filter!=null? (filter.accept(client)?client:null) : client;
		}
		
	}
	
	/**
	 * allows to filter the available SMF's
	 */
	public static interface StorageFactoryFilter{
		
		public boolean accept(EndpointReferenceType epr);
		
		public boolean accept(StorageFactoryClient smf);
	
	}
	
}
